package cn.hanghui.outapi.platform.alipay.config;

import cn.hanghui.outapi.platform.alipay.constant.AlipayFaceAuthParam;
import cn.hanghui.outapi.platform.alipay.constant.AlipayOpenApiProperties;
import cn.hanghui.outapi.platform.alipay.entity.rest.request.AlipayAppAuthCancelledMqRequest;
import cn.hanghui.outapi.platform.alipay.entity.rest.request.AlipayUserCreateOpenMqRequest;
import cn.hanghui.outapi.platform.alipay.entity.rest.request.AlipayUserDeleteOpenMqRequest;
import cn.hanghui.outapi.platform.alipay.service.AlipayOpenApiService;
import cn.hanghui.outapi.platform.tenant.service.TenantService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.msg.AlipayMsgClient;
import com.alipay.api.msg.MsgHandler;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

@Data
@Configuration
@Slf4j
@Component
public class AlipayConfig {

    @Autowired
    private TenantService tenantService;

    @Autowired
    @Lazy
    private AlipayOpenApiService alipayOpenApiService;

    /**
     * 支付宝人脸服务客户端
     *
     * @param alipayFaceAuthParam AlipayFaceParam
     * @return AlipayClient
     */
    @Bean(name = "alipayFaceAuthClient")
    @Qualifier(value = "alipayFaceAuthClient")
    public AlipayClient alipayFaceAuthClient(AlipayFaceAuthParam alipayFaceAuthParam) {
        return new DefaultAlipayClient(alipayFaceAuthParam.getGateway(),
                alipayFaceAuthParam.getAppid(),
                alipayFaceAuthParam.getPrivateKey(),
                "json", "utf-8", alipayFaceAuthParam.getAlipayPublicKey(), "RSA2");
    }

    /**
     * 2188
     *
     * @param alipayOpenApiProperties
     * @return
     */
    @Bean(name = "alipayOpenApiClient")
    @Qualifier(value = "alipayOpenApiClient")
    public AlipayClient alipayOpenApiClient(AlipayOpenApiProperties alipayOpenApiProperties) {
        return new DefaultAlipayClient(alipayOpenApiProperties.getALIPAY_SERVICE_URL(),
                alipayOpenApiProperties.getAppId(), alipayOpenApiProperties.getAlipayPrivateKey(),
                alipayOpenApiProperties.getFormat(), alipayOpenApiProperties.getCharset(),
                alipayOpenApiProperties.getAlipayPublicKey(), alipayOpenApiProperties.getSignType());
    }

//    /**
//     * 消息订阅 (航汇支付宝商户平台)
//     * ⚠️注意：非航汇支付宝商户平台部署时可注释掉
//     *
//     * @param alipayOpenApiProperties
//     * @return
//     */
//    @Bean(name = "alipayMsgClient2188")
//    @Qualifier(value = "alipayMsgClient2188")
//    public AlipayMsgClient alipayMsgClient2188(AlipayOpenApiProperties alipayOpenApiProperties) {
//        AlipayMsgClient alipayMsgClient = AlipayMsgClient.getInstance(alipayOpenApiProperties.getAppId());
//        try {
//            alipayMsgClient.setConnector(alipayOpenApiProperties.getOPENCHANNEL_SERVICE_URL());
//            alipayMsgClient.setSecurityConfig(alipayOpenApiProperties.getSignType(), alipayOpenApiProperties.getAlipayPrivateKey(), alipayOpenApiProperties.getAlipayPublicKey());
//            alipayMsgClient.setCharset(alipayOpenApiProperties.getCharset());
//            alipayMsgClient.setMessageHandler(new MsgHandler() {
//                /**
//                 * 客户端接收到消息后回调此方法
//                 *  @param  msgApi 接收到的消息的消息api名
//                 *  @param  msgId 接收到的消息的消息id
//                 *  @param  bizContent 接收到的消息的内容，json格式
//                 */
//                @Override
//                public void onMessage(String msgApi, String msgId, String bizContent) {
//                    log.info("--->客户端接收到消息后回调此方法响应参数msgApi:{}, msgId:{}, bizContent:{}", msgApi, msgId, bizContent);
//                    switch (msgApi) {
//                        case "alipay.open.iotvsp.usercreate.notify":
//                            AlipayUserCreateOpenMqRequest createOpenMqRequest = JSON.parseObject(bizContent, AlipayUserCreateOpenMqRequest.class);
//                            //TODO 执行入库成功或失败处理
//                            log.info("--->执行入库成功或失败处理响应参数:{}", JSONObject.toJSONString(createOpenMqRequest));
//                            alipayOpenApiService.notifyCreateUserState(createOpenMqRequest);
//                            break;
//                        case "alipay.open.iotvsp.userdelete.notify":
//                            AlipayUserDeleteOpenMqRequest deleteOpenMqRequest = JSON.parseObject(bizContent, AlipayUserDeleteOpenMqRequest.class);
//                            //TODO 执行出库成功或失败处理
//                            log.info("--->执行出库成功或失败处理响应参数:{}", JSONObject.toJSONString(deleteOpenMqRequest));
//                            alipayOpenApiService.notifyDelUserState(deleteOpenMqRequest);
//                            break;
//                        default:
//                            break;
//                    }
//                }
//            });
//            alipayMsgClient.connect();
//        } catch (Exception e) {
//            log.error("--->消息订阅websocket连接异常", e);
//        }
//        return alipayMsgClient;
//    }

}
