package cn.hanghui.outapi.platform.alipay.controller;

import cn.hanghui.outapi.platform.alipay.entity.dto.AlipayUserInfoReqDto;
import cn.hanghui.outapi.platform.alipay.entity.param.AlipayUserInfo;
import cn.hanghui.outapi.platform.alipay.service.AlipayFaceAuthService;
import cn.hanghui.outapi.platform.common.entity.rest.Result;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@Validated
@Api(tags = "支付宝人脸认证服务")
@RestController
@RequestMapping("alipayApi/faceAuth")
public class AlipayFaceAuthController {

    @Autowired
    private AlipayFaceAuthService alipayFaceAuthService;

    /**
     * 获取支付宝用户信息
     *
     * @param alipayUserInfoReqDto AlipayUserInfoReqDto
     * @return Result<AlipayUserInfo>
     */
    @ApiOperation("获取支付宝用户信息")
    @PostMapping("getAlipayUserInfo")
    public Result<AlipayUserInfo> getAlipayUserInfo(@RequestBody @Validated AlipayUserInfoReqDto alipayUserInfoReqDto) {
        log.info("--->获取支付宝用户信息原始入参:{}", JSONObject.toJSONString(alipayUserInfoReqDto));
        if (alipayUserInfoReqDto.getAuthCode() == null
                && alipayUserInfoReqDto.getSn() == null
                && alipayUserInfoReqDto.getFtoken() == null
                && alipayUserInfoReqDto.getAlipayUid() == null) {
            log.error("--->参数有误");
            return Result.ok(new AlipayUserInfo());
        }
        return Result.ok(alipayFaceAuthService.getAlipayUserInfo(alipayUserInfoReqDto));
    }

    /**
     * • appId为空时默认2188
     * • 会根据sn获取appId
     * • 然后选择对应网关sdk
     *
     * @param alipayUserInfoReqDto
     * @return
     */
    @ApiOperation("获取支付宝用户信息 (新版本 通用)")
    @PostMapping("/common/getAlipayUserInfo")
    public Result<AlipayUserInfo> getCommonAlipayUserInfo(@RequestBody @Validated AlipayUserInfoReqDto alipayUserInfoReqDto) {
        log.info("--->获取通用支付宝用户信息原始入参:{}", JSONObject.toJSONString(alipayUserInfoReqDto));
        if (alipayUserInfoReqDto.getAuthCode() == null
                && alipayUserInfoReqDto.getSn() == null
                && alipayUserInfoReqDto.getFtoken() == null
                && alipayUserInfoReqDto.getAlipayUid() == null) {
            log.error("--->参数有误");
            return Result.ok(new AlipayUserInfo());
        }
        return Result.ok(alipayFaceAuthService.getCommonAlipayUserInfo(alipayUserInfoReqDto));
    }

    @ApiOperation("获取支付宝用户信息(通用查询 一对多模型)")
    @PostMapping("/common/user/info/query")
    public Result<AlipayUserInfo> queryCommonUserInfo(@RequestBody @Validated AlipayUserInfoReqDto alipayUserInfoReqDto) {
        log.info("--->获取支付宝用户信息原始入参:{}", JSONObject.toJSONString(alipayUserInfoReqDto));
        if (alipayUserInfoReqDto.getAuthCode() == null
                && alipayUserInfoReqDto.getSn() == null
                && alipayUserInfoReqDto.getFtoken() == null
                && alipayUserInfoReqDto.getAlipayUid() == null) {
            log.error("--->获取用户信息原始入参参数有误");
            return Result.ok(new AlipayUserInfo());
        }
        return Result.ok(alipayFaceAuthService.queryCommonUserInfo(alipayUserInfoReqDto));
    }

    private Map<String, String> getParam(HttpServletRequest request) {
        Map<String, String> filterMap = new HashMap<>();
        try {
            request.setCharacterEncoding("utf-8");
            //遍历
            Map<String, String[]> map = request.getParameterMap();
            if (map.isEmpty()) {
                return filterMap;
            }
            for (String key : map.keySet()) {
                log.info(key + ":");
                // 获取值
                String[] values = map.get(key);
                if (values.length > 0) {
                    filterMap.put(key, values[0]);
                    for (String value : values) {
                        log.info(value + ":");
                    }
                }
            }

        } catch (Exception e) {
            log.error("--->获取get参数异常", e);
        }
        return filterMap;
    }


}
