package cn.hanghui.outapi.platform.common.service;

import com.alipay.api.AlipayClient;
import lombok.AllArgsConstructor;
import lombok.Data;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Arrays;

/**
 * @author shisl
 * @package cn.hanghui.outapi.platform.common.service
 * @class BaseService
 * @date 2023/7/27 下午1:42
 * @description
 */
@Service
public class BaseService {

    //2188
    @Value("${alipay.open.api.appid}")
    private String appId2188;
    @Value("${alipay.open.api.private-key}")
    private String privateKey2188;
    @Value("${alipay.open.api.componentOutId}")
    private String componentOutId2188;
    @Value("${alipay.open.api.isvPid}")
    private String isvPid2188;
    @Value("${alipay.open.api.orgOutId}")
    private String orgOutId2188;

    @Value("${alipay.open.api.deviceComponentOutId}")
    private String verifiedComponentOutId;
    @Value("${alipay.open.api.anonymousComponentOutId}")
    private String anonymousComponentOutId;
    @Value("${alipay.open.api.componentOutIdScenic}")
    private String commonComponentOutId;

    // 8975
    @Value("${alipay.open.api.appId8975}")
    private String appId8975;
    @Value("${alipay.open.api.private-key8975}")
    private String privateKey8975;

    @Resource
    private AlipayClient alipayOpenApiClient;

    /**
     * 获取网关 默认2188
     *
     * @param appId
     * @return
     */
    public AlipayClient getAlipayClient(String appId) {

        if (appId == null || appId2188.equals(appId)) {
            return alipayOpenApiClient;
        }

        return alipayOpenApiClient;
    }

    /**
     * 获取组件编码
     *
     * @param componentType
     * @return
     */
    public String getComponentOutId(String componentType) {
        if (componentType == null || componentType.equals("anonymous")) {
            return anonymousComponentOutId;
        }

        if (componentType.equals("verified")) {
            return verifiedComponentOutId;
        }
        if (componentType.equals("common")) {
            return commonComponentOutId;
        }

        return commonComponentOutId;
    }

    /**
     * 获取isv信息参数 出入库
     * • 默认2188
     *
     * @param appId
     * @return
     */
    public IsvInfo getIsvInfo(String appId) {
        IsvInfo isvInfo = new IsvInfo(componentOutId2188, isvPid2188, orgOutId2188);
        if (appId == null || appId2188.equals(appId)) {
            return isvInfo;
        }

        return isvInfo;
    }

    /**
     * key8
     *
     * @param appId
     * @return
     */
    public String getKey8PrivateKey(String appId) {
        if (appId == null || appId8975.equals(appId)) {
            return privateKey8975;
        }
        if (appId2188.equals(appId)) {
            return privateKey2188;
        }
        return privateKey8975;
    }



    /**
     * 校验是否能获取ftoken
     * appId白名单
     *
     * @param appId
     * @return
     */
    public Boolean checkFtoken(String appId) {
        if (appId == null) {
            return Boolean.FALSE;
        }
        return Arrays.asList("2019040163757082", appId2188).contains(appId);
    }

    @Data
    @AllArgsConstructor
    public class IsvInfo {
        private String componentOutId;
        private String isvPid;
        private String orgOutId;
    }


}
