package cn.hanghui.outapi.platform.tenant.entity.po;

import cn.hanghui.outapi.platform.common.entity.po.Base;
import cn.hanghui.outapi.platform.tenant.entity.dto.ManualInIdCardAuthDto;
import cn.hanghui.outapi.platform.tenant.typehandle.ManualInIdCardAuthHandler;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.Date;

/**
 * 设备
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "device", autoResultMap = true)
public class Device extends Base {

    /**
     * s设备编号
     */
    private String sn;

    /**
     * 公司
     */
    private String company;

    /**
     * 商户平台id
     */
    private Long tenantId;

    /**
     * 场景id
     */
    private Long sceneId;

    /**
     * 用户库id
     */
    private Long userLibId;

    /**
     * 模式 mode: public/close/elasticity 三个参数分别代表 开放/封闭/弹性
     */
    private String mode;

    /**
     * 授权状态  0：未授权 1：已授权
     */
    private Boolean status = Boolean.FALSE;

    /**
     * 第一次授权时间
     */
    private Date firstAuthTime;

    /**
     * ==============================20230920设备配置额外增加====================================
     */
    /**
     * 是否启用支付宝人脸能力 默认：true
     */
    private Boolean useAlipayFace;

    /**
     * 联动平台地址
     */
    private String linkageAds;

    /**
     * 刷脸成功结果页时长 1-10 默认值2
     */
    private Integer successPageTime;

    /**
     * 刷脸失败结果页时长 1-10 默认值5
     */
    private Integer failPageTime;

    /**
     * zoloConfig摄像头配置 ZoloConfigDto
     */
    private String zoloConfig;

    /**
     * 是否启用身份证功能 默认：true
     */
    private Boolean useIdcard;

    /**
     * 是否人证比对 默认：false
     */
    private Boolean usePersonCard;

    /**
     * 是否启用二维码功能 默认：true
     */
    private Boolean useQrcode;

    /**
     * 是否启用联动平台 默认：true
     */
    private Boolean useLinkage;

    /**
     * 刷脸模式: 点击/自动 click/auto  默认 auto
     */
    private String faceMode;

    /**
     * 点击刷脸按钮文本  默认 点击开始刷脸核验
     */
    private String clickButtonText;

    /**
     * 是否抓拍人脸 默认：false
     */
    private Boolean isCaptureFace;

    /**
     * need1V1是否使用支付宝刷证提示页 默认：true
     */
    private Boolean isNeed1V1;

    /**
     * 开门信号: 继电器/RS232 default/RS232 默认：default
     */
    private String useSerialPort;

    /**
     * RS232默认信号
     */
    private String signalNo;

    /**
     * 补光灯: 自动 auto 默认:auto
     */
    private String lightMode;

    /**
     * 语音音量 0-100 默认值80
     */
    private Integer volume;

    /**
     * 成功语音 默认: 请通行
     */
    private String sucVoice;

    /**
     * 成功文字 默认: 请通行
     */
    private String sucText;

    /**
     * 失败语音 默认: 比对失败,请联系现场工作人员
     */
    private String failVoice;

    /**
     * 失败文字 默认: 比对失败,请联系现场工作人员
     */
    private String failText;

    /**
     * 隐藏底部导航栏 默认: false
     */
    private Boolean hideNavigationBar;

    /**
     * 禁止下拉通知栏 默认: false
     */
    private Boolean forbidPullDown;

    /**
     * 首页图片
     */
    private String homePageUrl;

    /**
     * 过渡图片
     */
    private String transitionPageUrl;


    /**
     * ============================20231007添加===========================
     */
    /**
     * 设备在线状态心跳地址
     */
    private String heartTrackingUrl;

    /**
     * 设备在线状态心跳地址
     */
    private String transitPushUrl;

    /**
     * 是否显示顶部栏
     */
    private Boolean showTopBar;
    /**
     * 顶部栏左边显示内容
     */
    private String leftTopBarText;
    /**
     * 顶部栏右边显示内容
     */
    private String rightTopBarText;

    /**
     * 通行方式: 通用(normal) 进站刷脸全量通行(inStation)  出站刷脸全量通行(outStation)
     */
    private String passWay;

    /**
     * ============================20231110添加===========================
     */
    /**
     * 是否开启年龄限制
     */
    private Boolean isOpenAgeLimit;

    /**
     * 限制区间-开始年龄
     */
    private Integer limitSectionStart;

    /**
     * 限制区间-结束年龄
     */
    private Integer limitSectionEnd;

    /**
     * 禁止进入文本
     */
    private String ageForbidText;

    /**
     * 设备管理员密码
     */
    private String mpassword;
    /**
     * ============================20231128添加===========================
     */
    /*
    *  是否开启护照机
    * */
    private Boolean isOpenPassport;
    /*
     *  是否推送门磁信号
     * */
    @TableField("is_push_gm")
    private Boolean isPushGM;
    /*
     *  门磁信号推送地址
     * */
    @TableField("gm_address")
    private String GMAddress;
    /*
     *  门磁信号推送频率
     * */
    @TableField("gm_frequency")
    private Integer GMFrequency;
    /*
     *  应用是否自启动
     * */
    private Boolean autoLaunch;
    /**
     * ============================20231215添加===========================
     */
    @TableField("use_entry_detector")
    private Boolean useEntryDetector;



    @ApiModelProperty(value = "购买方公司id")
    private Long purchaserId;

    @ApiModelProperty(value = "应用类型id")
    private Long applicationTypeId;

    @ApiModelProperty(value = "设备型号(硬件)id")
    private Long deviceModelHardwareId;

    @ApiModelProperty(value = "设备型号(环境)id")
    private Long deviceModelEnvironmentId;

    @ApiModelProperty(value = "系统类型 1-蚂蚁OS 2-非蚂蚁OS")
    private Integer systemTypeCode;

    /**
     * ============================20240103添加===========================
     */
    @ApiModelProperty(value = "在线日志推送地址")
    private String logPushUrl;
    /**
     * @see cn.hanghui.outapi.platform.tenant.entity.enums.CameraTypeEnum
     */
    @ApiModelProperty(value = "摄像头类型 IR-双目  3D-3D结构光 ")
    private String cameraType;
    /**
     * @see cn.hanghui.outapi.platform.tenant.entity.enums.CameraBrandEnum
     */
    @ApiModelProperty(value = "品牌 Normal-通用   Mario-马里奥")
    private String cameraBrand;
    /**
     * ============================20240105添加===========================
     */
    /**
     * @see cn.hanghui.outapi.platform.tenant.entity.enums.EntryFaceModeEnum
     */
    @ApiModelProperty(value = "进场刷脸模式")
    private String entryFaceMode;

    /**
     * ============================20240228添加===========================
     */
    /**
     * @see cn.hanghui.outapi.platform.tenant.entity.enums.UserOutputModeEnum
     */
    @ApiModelProperty(value = "用户信息输出模式  NoOutput:不输出（默认）， AnalogSignal:阅读器模拟信号")
    private String userOutputMode;

    @ApiModelProperty(value = "身份证阅读器品牌 IdCardReaderBrandEnum")
    private Integer idCardReaderBrand;

    /**
     * ============================20240315添加===========================
     */
    @ApiModelProperty(value = "是否设备实名入库 默认false")
    private Boolean authVerifiedFacein;

    @ApiModelProperty(value = "是否开启手动输入证件号 默认false")
    private Boolean manualInIdCard;

    @ApiModelProperty(value = "开启手动输入证件号的密钥信息")
    @TableField(value = "manual_in_id_card_auth",typeHandler = ManualInIdCardAuthHandler.class)
    private ManualInIdCardAuthDto manualInIdCardAuth;
    /**
     * ============================20240327添加===========================
     */
    /**@see cn.hanghui.outapi.platform.tenant.entity.enums.LinkageCheckModeEnum*/
    @ApiModelProperty(value = "联动平台校验模式 1-通用 2-预检查模式")
    private Integer linkageCheckMode;
    /**@see cn.hanghui.outapi.platform.tenant.entity.enums.ResultPageTemplateEnum*/
    @ApiModelProperty(value = "结果页模版 1-通用版 2-悬浮版")
    private Integer resultPageTemplate;

    /**
     * ============================20240423添加===========================
     */
    /**@see cn.hanghui.outapi.platform.tenant.entity.enums.RelayOpenModelEnum*/
    @ApiModelProperty(value = "继电器开门模式 1-开-闭信号 2-开-门磁-闭信息")
    private Integer relayOpenModel;
    @ApiModelProperty(value = "开闭时间间隔（s）")
    private Integer openCloseInterval;
    @ApiModelProperty(value = "信号间间隔（s）")
    private Integer signalInterval;
    @ApiModelProperty(value = "门磁超时时间（s）")
    private Integer gateTimeOut;

    /**
     * ============================20240503添加===========================
     */
    @ApiModelProperty(value = "是否解析身份证照片 0-关闭 1-开启 默认-1")
    private Integer analysisPhoto;
    @ApiModelProperty(value = "是否解析身份证指纹 0-关闭 1-开启 默认-0")
    private Integer analysisFingerprint;
    /**
     * ============================20240506添加===========================
     */
    @ApiModelProperty(value = "同一人刷证间隔时间 取值范围0-20 默认-3")
    private Integer idCardInterval;

    @ApiModelProperty(value = "刷脸信息推送地址")
    private String pushFaceInfoUrl;


}
